@@ -140,20 +140,17 @@
   (let ((state (chacha-state context))
         (keystream-buffer (chacha-keystream-buffer context))
         (keystream-buffer-remaining (chacha-keystream-buffer-remaining context))
-        (core-function (chacha-core-function context))
-        (remaining-keystream (make-array 64 :element-type '(unsigned-byte 8))))
+        (core-function (chacha-core-function context)))
     (declare (type chacha-state state)
-             (type chacha-keystream-buffer keystream-buffer remaining-keystream)
+             (type chacha-keystream-buffer keystream-buffer)
              (type (integer 0 64) keystream-buffer-remaining)
-             (type function core-function)
-             (dynamic-extent remaining-keystream))
+             (type function core-function))
     (unless (zerop length)
       (unless (zerop keystream-buffer-remaining)
         (let ((size (min length keystream-buffer-remaining)))
           (declare (type (integer 0 64) size))
-          (replace remaining-keystream keystream-buffer
-                   :end1 size :start2 (- 64 keystream-buffer-remaining))
-          (xor-block size remaining-keystream plaintext plaintext-start
+          (xor-block size keystream-buffer (- 64 keystream-buffer-remaining)
+                     plaintext plaintext-start
                      ciphertext ciphertext-start)
           (decf keystream-buffer-remaining size)
           (decf length size)
@@ -166,11 +163,11 @@
                              (mod32+ (aref state 12) 1)))
             (setf (aref state 13) (mod32+ (aref state 13) 1)))
           (when (<= length 64)
-            (xor-block length keystream-buffer plaintext plaintext-start
+            (xor-block length keystream-buffer 0 plaintext plaintext-start
                        ciphertext ciphertext-start)
             (setf (chacha-keystream-buffer-remaining context) (- 64 length))
             (return-from chacha-crypt (values)))
-          (xor-block 64 keystream-buffer plaintext plaintext-start
+          (xor-block 64 keystream-buffer 0 plaintext plaintext-start
                      ciphertext ciphertext-start)
           (decf length 64)
           (incf ciphertext-start 64)
