@@ -3,7 +3,7 @@
 
 (in-package :crypto)
 
-(defclass cmac ()
+(defclass cmac (mac)
   ((cipher :reader cmac-cipher :initarg :cipher)
    (subkey1 :reader cmac-subkey1 :initarg :subkey1
             :type (simple-array (unsigned-byte 8) (*)))
@@ -50,23 +50,46 @@
   (reinitialize-instance (cmac-cipher mac) :key key :mode :ecb)
   mac)
 
-(defun update-cmac (cmac sequence
-                    &key (start 0) end)
-  (declare (type (simple-array (unsigned-byte 8) (*)) sequence))
-  (declare (type index start))
-  (declare (type (or index null) end))
-  (do ((end (or end (length sequence)))
-       (buffer (cmac-buffer cmac))
-       (length (length (cmac-buffer cmac)))
-       (i start (1+ i)))
-      ((>= i end) cmac)
+(defun update-cmac (cmac sequence &key (start 0) (end (length sequence)))
+  (declare (type (simple-array (unsigned-byte 8) (*)) sequence)
+           (type index start end)
+           (optimize (speed 3) (space 0) (safety 0) (debug 0)))
+  (let* ((cipher (cmac-cipher cmac))
+         (encryption-function (encrypt-function cipher))
+         (buffer (cmac-buffer cmac))
+         (buffer-index (cmac-buffer-index cmac))
+         (block-length (length buffer))
+         (remaining (- end start)))
     (declare (type (simple-array (unsigned-byte 8) (*)) buffer))
-    (when (= length (cmac-buffer-index cmac))
-      (setf (cmac-buffer-index cmac) 0)
-      (encrypt-in-place (cmac-cipher cmac) buffer))
-    (setf (aref buffer (cmac-buffer-index cmac))
-          (logxor (aref buffer (cmac-buffer-index cmac)) (aref sequence i)))
-    (incf (cmac-buffer-index cmac))))
+
+    (when (< 0 buffer-index block-length)
+      (dotimes (i (min remaining (- block-length buffer-index)))
+        (setf (aref buffer buffer-index) (logxor (aref buffer buffer-index)
+                                                 (aref sequence start)))
+        (incf buffer-index)
+        (incf start)
+        (decf remaining)))
+
+    (when (and (= buffer-index block-length)
+               (plusp remaining))
+      (funcall encryption-function cipher buffer 0 buffer 0)
+      (setf buffer-index 0))
+
+    (loop while (> remaining block-length) do
+      (xor-block block-length buffer 0 sequence start buffer 0)
+      (funcall encryption-function cipher buffer 0 buffer 0)
+      (incf start block-length)
+      (decf remaining block-length))
+
+    (loop while (plusp remaining) do
+      (setf (aref buffer buffer-index) (logxor (aref buffer buffer-index)
+                                               (aref sequence start)))
+      (incf buffer-index)
+      (incf start)
+      (decf remaining))
+
+    (setf (cmac-buffer-index cmac) buffer-index)
+    (values)))
 
 (defun cmac-digest (cmac)
   (let* ((block-length (length (cmac-buffer cmac)))
@@ -78,7 +101,7 @@
                (setf (aref x (cmac-buffer-index cmac))
                      (logxor (aref x (cmac-buffer-index cmac)) #x80))
                (cmac-subkey2 cmac)))))
-    (xor-block block-length L x 0 x 0)
+    (xor-block block-length L 0 x 0 x 0)
     (encrypt-in-place (cmac-cipher cmac) x)
     x))
 
