@@ -37,6 +37,9 @@
    #:mac-supported-p #:list-all-macs
    #:mode-supported-p #:list-all-modes
    #:block-length #:digest-length #:key-lengths
+   #:keystream-position
+   #:authenticated-encryption-mode-supported-p
+   #:list-all-authenticated-encryption-modes
 
    ;; high-level block cipher operators
    #:make-cipher #:encrypt #:decrypt #:encrypt-in-place #:decrypt-in-place
@@ -44,6 +47,15 @@
    ;; arguments to (MAKE-CIPHER ... :MODE X)
    #:ecb #:cbc #:ctr #:ofb #:cfb #:stream
 
+   ;; block cipher padding
+   #:pkcs7 #:ansi-x923 #:iso-7816-4
+
+   ;; authenticated encryption
+   #:make-authenticated-encryption-mode
+   #:process-associated-data
+   #:produce-tag
+   #:gcm #:etm
+
    ;; KDFs
    #:pbkdf1 #:pbkdf2 #:scrypt-kdf #:argon2i
    #:make-kdf #:derive-key
@@ -95,7 +107,8 @@
    ;; conditions
    #:ironclad-error #:initialization-vector-not-supplied
    #:invalid-initialization-vector #:invalid-key-length
-   #:unsupported-cipher #:unsupported-mode #:unsupported-digest
+   #:unsupported-cipher #:unsupported-mode #:unsupported-padding
+   #:unsupported-digest
    #:unsupported-kdf #:unsupported-scrypt-cost-factors
    #:unsupported-argon2i-parameters
    #:insufficient-buffer-space #:invalid-padding
@@ -105,6 +118,8 @@
    #:missing-message-parameter #:missing-signature-parameter
    #:incompatible-keys #:invalid-curve-point
    #:invalid-public-key-length #:oaep-decoding-error
+   #:unsupported-authenticated-encryption-mode
+   #:bad-authentication-tag
 
    ;; utilities
    #:byte-array-to-hex-string #:hex-string-to-byte-array
@@ -133,17 +148,20 @@
            #:skein1024 #:skein1024/384 #:skein1024/512
            #:sha3 #:sha3/384 #:sha3/256 #:sha3/224
            #:shake128 #:shake256
+           #:keccak #:keccak/384 #:keccak/256 #:keccak/224
            #:groestl #:groestl/384 #:groestl/256 #:groestl/224
            #:blake2 #:blake2/384 #:blake2/256 #:blake2/160
            #:blake2s #:blake2s/224 #:blake2s/160 #:blake2s/128
-           #:jh #:jh/384 #:jh/256 #:jh/224)
+           #:jh #:jh/384 #:jh/256 #:jh/224
+           #:streebog #:streebog/256)
   ;; supported macs
-  (:export #:blake2-mac #:blake2s-mac #:cmac #:hmac #:poly1305 #:skein-mac)
+  (:export #:blake2-mac #:blake2s-mac #:cmac #:hmac #:poly1305 #:skein-mac
+           #:gmac)
   ;; supported block ciphers
   (:export #:blowfish #:tea #:xtea #:square #:rc2 #:rc5 #:rc6 #:des #:3des
            #:aes #:twofish #:cast5 #:idea #:misty1 #:xor
            #:threefish256 #:threefish512 #:threefish1024
-           #:serpent #:camellia #:seed #:aria)
+           #:serpent #:camellia #:seed #:aria #:kuznyechik)
   ;; supported stream ciphers
   (:export #:arcfour
            #:salsa20 #:salsa20/12 #:salsa20/8
@@ -151,3 +169,58 @@
            #:chacha #:chacha/12 #:chacha/8
            #:xchacha #:xchacha/12 #:xchacha/8
            #:sosemanuk))
+
+(in-package :crypto)
+
+;;; easy-to-type readmacro for creating s-boxes and the like
+
+(defun array-reader (stream subchar arg)
+  (declare (ignore subchar))
+  (let ((array-data (read stream nil stream nil))
+        (array-element-type `(unsigned-byte ,arg)))
+    ;; FIXME: need to make this work for multi-dimensional arrays
+    `(make-array ,(length array-data) :element-type ',array-element-type
+                :initial-contents ',array-data)))
+
+(defparameter *ironclad-readtable*
+  (let ((readtable (copy-readtable nil)))
+    (set-dispatch-macro-character #\# #\@ #'array-reader readtable)
+    readtable))
+
+(defmacro in-ironclad-readtable ()
+  `(eval-when (:compile-toplevel :load-toplevel :execute)
+     (setq *readtable* *ironclad-readtable*)))
+
+(defun ironclad-implementation-features ()
+  #+sbcl
+  (list* sb-c:*backend-byte-order*
+         (if (= sb-vm:n-word-bits 32)
+             :32-bit
+             :64-bit)
+         :ironclad-fast-mod32-arithmetic
+         (when (member :x86-64 *features*)
+           '(:ironclad-fast-mod64-arithmetic)))
+  #+cmu
+  (list (c:backend-byte-order c:*target-backend*)
+        (if (= vm:word-bits 32)
+            :32-bit
+            :64-bit)
+        :ironclad-fast-mod32-arithmetic)
+  #+lispworks
+  (list ;; Disable due to problem reports from Lispworks users and
+        ;; non-obviousness of the fix.
+        #+nil
+        (when (not (member :lispworks4 *features*))
+          '(:ironclad-md5-lispworks-int32)))
+  #+openmcl
+  (list* (when (member :x86-64 *features*)
+           '(:ironclad-fast-mod32-arithmetic
+             :ironclad-fast-mod64-arithmetic)))
+  #+ecl
+  (list :ironclad-fast-mod32-arithmetic
+        :ironclad-fast-mod64-arithmetic)
+  #-(or sbcl cmu lispworks openmcl ecl)
+  nil)
+
+(dolist (f (ironclad-implementation-features))
+  (pushnew f *features*))
