@@ -53,6 +53,13 @@
   (values)
   (any) :overwrite-fndb-silently t)
 
+#+x86-64
+(defknown ironclad::pclmulqdq-support-p
+  ()
+  (boolean)
+  (any)
+  :overwrite-fndb-silently t)
+
 #+x86-64
 (defknown ironclad::aes-ni-support-p
   ()
@@ -108,4 +115,41 @@
           (unsigned-byte 32))
   (any)
   :overwrite-fndb-silently t)
+
+#+x86-64
+(defknown ironclad::gmac-mul-fast
+  ((simple-array (unsigned-byte 8) (*))
+   (simple-array (unsigned-byte 8) (*)))
+  (values)
+  (any)
+  :overwrite-fndb-silently t)
+
+#+x86-64
+(defknown ironclad::xor128
+  ((simple-array (unsigned-byte 8) (*))
+   (integer 0 #.array-dimension-limit)
+   (simple-array (unsigned-byte 8) (*))
+   (integer 0 #.array-dimension-limit)
+   (simple-array (unsigned-byte 8) (*))
+   (integer 0 #.array-dimension-limit))
+  (values)
+  (any)
+  :overwrite-fndb-silently t)
+
+#+x86-64
+(defknown ironclad::mov128
+  ((simple-array (unsigned-byte 8) (*))
+   (integer 0 #.array-dimension-limit)
+   (simple-array (unsigned-byte 8) (*))
+   (integer 0 #.array-dimension-limit))
+  (values)
+  (any)
+  :overwrite-fndb-silently t)
+
+(defknown ironclad::inc-counter-block
+  ((integer 0 #.most-positive-fixnum)
+   (simple-array (unsigned-byte 8) (*)))
+  (values)
+  (any)
+  :overwrite-fndb-silently t)
 );#+sbcl
