@@ -112,6 +112,23 @@
   (cipher-test-guts cipher-name :stream hexkey hexinput hexoutput
                     (list :initialization-vector hexiv)))
 
+(defun keystream-test (cipher-name key iv keystream)
+  (let* ((mode (if (= 1 (crypto:block-length cipher-name)) :stream :ctr))
+         (cipher (crypto:make-cipher cipher-name :key key :mode mode :initialization-vector iv))
+         (buffer (make-array 1000 :element-type '(unsigned-byte 8) :initial-element 0)))
+    (crypto:keystream-position cipher 100)
+    (crypto:encrypt-in-place cipher buffer :start 100 :end 213)
+    (crypto:keystream-position cipher 500)
+    (crypto:encrypt-in-place cipher buffer :start 500 :end 1000)
+    (crypto:keystream-position cipher 213)
+    (crypto:encrypt-in-place cipher buffer :start 213 :end 500)
+    (crypto:keystream-position cipher 0)
+    (crypto:encrypt-in-place cipher buffer :end 100)
+    (crypto:keystream-position cipher 765)
+    (when (or (/= (crypto:keystream-position cipher) 765)
+              (mismatch buffer keystream))
+      (error "getting/setting key stream position failed for ~A on key ~A" cipher-name key))))
+
 #+(or lispworks sbcl cmucl openmcl allegro abcl ecl clisp)
 (defun stream-mode-test/stream (cipher-name hexkey hexinput hexoutput)
   (cipher-stream-test-guts cipher-name :stream hexkey hexinput hexoutput))
@@ -125,14 +142,16 @@
   (list (cons :ecb-mode-test 'ecb-mode-test)
         (cons :ecb-tweak-mode-test 'ecb-tweak-mode-test)
         (cons :stream-mode-test 'stream-mode-test)
-        (cons :stream-nonce-mode-test 'stream-nonce-mode-test)))
+        (cons :stream-nonce-mode-test 'stream-nonce-mode-test)
+        (cons :keystream-test 'keystream-test)))
 
 #+(or lispworks sbcl cmucl openmcl allegro abcl ecl clisp)
 (defparameter *cipher-stream-tests*
   (list (cons :ecb-mode-test 'ignore-test)
         (cons :ecb-tweak-mode-test 'ignore-test)
         (cons :stream-mode-test 'stream-mode-test/stream)
-        (cons :stream-nonce-mode-test 'stream-nonce-mode-test/stream)))
+        (cons :stream-nonce-mode-test 'stream-nonce-mode-test/stream)
+        (cons :keystream-test 'ignore-test)))
 
 
 ;;; encryption mode consistency checking
@@ -157,8 +176,35 @@
         (error "decryption failed for ~A on key ~A, input ~A, output ~A"
                cipher-name key output input)))))
 
+(defun mode-padding-test (mode cipher-name padding key iv input output)
+  (let ((cipher (crypto:make-cipher cipher-name
+                                    :key key
+                                    :mode mode
+                                    :initialization-vector iv
+                                    :padding padding))
+        (buffer1 (make-array (length input) :element-type '(unsigned-byte 8)))
+        (buffer2 (make-array (length output) :element-type '(unsigned-byte 8))))
+    (crypto:encrypt cipher input buffer2 :handle-final-block t)
+    (when (mismatch buffer2 output)
+      (error "encryption failed for ~A on key ~A, input ~A, output ~A"
+             cipher-name key input output))
+    (reinitialize-instance cipher
+                           :key key
+                           :mode mode
+                           :initialization-vector iv
+                           :padding padding)
+    (crypto:decrypt cipher output buffer1 :handle-final-block t)
+    (when (mismatch buffer1 input)
+      (error "decryption failed for ~A on key ~A, input ~A, output ~A"
+             cipher-name key output input))))
+
 (defparameter *mode-tests*
-  (list (cons :mode-test 'mode-test)))
+  (list (cons :mode-test 'mode-test)
+        (cons :mode-padding-test 'ignore-test)))
+
+(defparameter *mode-padding-tests*
+  (list (cons :mode-test 'ignore-test)
+        (cons :mode-padding-test 'mode-padding-test)))
 
 
 ;;; digest testing routines
@@ -294,7 +340,11 @@
                     (crypto:update-mac mac data)
                     (crypto:produce-mac mac))))
     (declare (ignorable result1))
-    (reinitialize-instance mac :key key)
+    (cond
+      ((typep mac 'ironclad:gmac)
+       (reinitialize-instance mac :key key :initialization-vector (car (last args))))
+      (t
+       (reinitialize-instance mac :key key)))
     (let ((result2 (progn
                      (crypto:update-mac mac data)
                      (crypto:produce-mac mac))))
@@ -536,3 +586,106 @@
   (list (cons :curve25519-dh-test 'curve25519-dh-test)
         (cons :curve448-dh-test 'curve448-dh-test)
         (cons :elgamal-dh-test 'elgamal-dh-test)))
+
+
+;;; authenticated encryption testing routines
+
+(defun aead-test (mode-name input ad output tag &rest args)
+  (let* ((parameters (case mode-name
+                       ((:gcm gcm crypto:gcm)
+                        (list :cipher-name (car args)
+                              :key (cadr args)
+                              :initialization-vector (caddr args)))
+                       ((:etm etm crypto:etm)
+                        (destructuring-bind (cipher-name ckey mode iv mac-name mkey mparam) args
+                          (let ((cipher (crypto:make-cipher cipher-name
+                                                            :key ckey
+                                                            :mode mode
+                                                            :initialization-vector iv))
+                                (mac (if mparam
+                                         (crypto:make-mac mac-name mkey mparam)
+                                         (crypto:make-mac mac-name mkey))))
+                            (list :cipher cipher :mac mac))))))
+         (ae (apply #'crypto:make-authenticated-encryption-mode mode-name parameters))
+         (ciphertext (crypto:encrypt-message ae input :associated-data ad)))
+    (when (or (mismatch ciphertext output)
+              (mismatch (crypto:produce-tag ae) tag))
+      (error "encryption failed for ~A, input ~A, output ~A" mode-name input output))
+    (setf parameters (case mode-name
+                       ((:gcm gcm crypto:gcm)
+                        parameters)
+                       ((:etm etm crypto:etm)
+                        (destructuring-bind (cipher-name ckey mode iv mac-name mkey mparam) args
+                          (let ((cipher (crypto:make-cipher cipher-name
+                                                            :key ckey
+                                                            :mode mode
+                                                            :initialization-vector iv))
+                                (mac (if mparam
+                                         (crypto:make-mac mac-name mkey mparam)
+                                         (crypto:make-mac mac-name mkey))))
+                            (list :cipher cipher :mac mac))))))
+    (apply #'reinitialize-instance ae :tag tag parameters)
+    (let ((plaintext (crypto:decrypt-message ae output :associated-data ad)))
+      (when (or (mismatch plaintext input)
+                (mismatch (crypto:produce-tag ae) tag))
+        (error "decryption failed for ~A, input ~A, output ~A" mode-name input output)))))
+
+(defun aead-test/incremental (mode-name input ad output tag &rest args)
+  (let* ((parameters (case mode-name
+                       ((:gcm gcm crypto:gcm)
+                        (list :cipher-name (car args)
+                              :key (cadr args)
+                              :initialization-vector (caddr args)))
+                       ((:etm etm crypto:etm)
+                        (destructuring-bind (cipher-name ckey mode iv mac-name mkey mparam) args
+                          (let ((cipher (crypto:make-cipher cipher-name
+                                                            :key ckey
+                                                            :mode mode
+                                                            :initialization-vector iv))
+                                (mac (if mparam
+                                         (crypto:make-mac mac-name mkey mparam)
+                                         (crypto:make-mac mac-name mkey))))
+                            (list :cipher cipher :mac mac))))))
+         (ae (apply #'crypto:make-authenticated-encryption-mode mode-name parameters))
+         (plaintext (make-array (length input) :element-type '(unsigned-byte 8)))
+         (ciphertext (make-array (length output) :element-type '(unsigned-byte 8))))
+    (dotimes (i (length ad))
+      (crypto:process-associated-data ae ad :start i :end (1+ i)))
+    (dotimes (i (length input))
+      (crypto:encrypt ae input ciphertext
+                      :plaintext-start i :plaintext-end (1+ i)
+                      :ciphertext-start i
+                      :handle-final-block (= i (1- (length input)))))
+    (when (or (mismatch ciphertext output)
+              (mismatch (crypto:produce-tag ae) tag))
+      (error "encryption failed for ~A, input ~A, output ~A" mode-name input output))
+    (setf parameters (case mode-name
+                       ((:gcm gcm crypto:gcm)
+                        parameters)
+                       ((:etm etm crypto:etm)
+                        (destructuring-bind (cipher-name ckey mode iv mac-name mkey mparam) args
+                          (let ((cipher (crypto:make-cipher cipher-name
+                                                            :key ckey
+                                                            :mode mode
+                                                            :initialization-vector iv))
+                                (mac (if mparam
+                                         (crypto:make-mac mac-name mkey mparam)
+                                         (crypto:make-mac mac-name mkey))))
+                            (list :cipher cipher :mac mac))))))
+    (apply #'reinitialize-instance ae :tag tag parameters)
+    (dotimes (i (length ad))
+      (crypto:process-associated-data ae ad :start i :end (1+ i)))
+    (dotimes (i (length output))
+      (crypto:decrypt ae output plaintext
+                      :ciphertext-start i :ciphertext-end (1+ i)
+                      :plaintext-start i
+                      :handle-final-block (= i (1- (length output)))))
+    (when (or (mismatch plaintext input)
+              (mismatch (crypto:produce-tag ae) tag))
+      (error "decryption failed for ~A, input ~A, output ~A" mode-name input output))))
+
+(defparameter *authenticated-encryption-tests*
+  (list (cons :aead-test 'aead-test)))
+
+(defparameter *authenticated-encryption-incremental-tests*
+  (list (cons :aead-test 'aead-test/incremental)))
